package org.opencdmp.filetransformer.rda.controller;

import gr.cite.tools.auditing.AuditService;
import gr.cite.tools.logging.LoggerService;
import gr.cite.tools.logging.MapLogEntry;
import org.opencdmp.commonmodels.models.FileEnvelopeModel;
import org.opencdmp.commonmodels.models.description.DescriptionModel;
import org.opencdmp.commonmodels.models.dmp.DmpModel;
import org.opencdmp.filetransformer.rda.audit.AuditableAction;
import org.opencdmp.filetransformerbase.interfaces.FileTransformerClient;
import org.opencdmp.filetransformerbase.interfaces.FileTransformerConfiguration;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.AbstractMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/api/file-transformer")
public class FileTransformerController implements org.opencdmp.filetransformerbase.interfaces.FileTransformerController {
    private static final LoggerService logger = new LoggerService(LoggerFactory.getLogger(FileTransformerController.class));

    private final FileTransformerClient fileTransformerExecutor;
    private final AuditService auditService;

    @Autowired
    public FileTransformerController(FileTransformerClient fileTransformerExecutor, AuditService auditService) {
	    this.fileTransformerExecutor = fileTransformerExecutor;
	    this.auditService = auditService;
    }

    public FileEnvelopeModel exportDmp(@RequestBody DmpModel dmpDepositModel, @RequestParam(value = "format",required = false)String format) throws Exception {
        logger.debug(new MapLogEntry("exportDmp " + DmpModel.class.getSimpleName()).And("dmpDepositModel", dmpDepositModel).And("format", format));

        FileEnvelopeModel model = fileTransformerExecutor.exportDmp(dmpDepositModel, format);

        this.auditService.track(AuditableAction.FileTransformer_ExportDmp, Map.ofEntries(
                new AbstractMap.SimpleEntry<String, Object>("dmpDepositModel", dmpDepositModel),
                new AbstractMap.SimpleEntry<String, Object>("format", format)
        ));
        return model;
    }

    public FileEnvelopeModel exportDescription(@RequestBody DescriptionModel descriptionModel, @RequestParam(value = "format",required = false)String format) throws Exception {
        logger.debug(new MapLogEntry("exportDescription " + DescriptionModel.class.getSimpleName()).And("descriptionModel", descriptionModel).And("format", format));

        FileEnvelopeModel model = fileTransformerExecutor.exportDescription(descriptionModel, format);

        this.auditService.track(AuditableAction.FileTransformer_ExportDescription, Map.ofEntries(
                new AbstractMap.SimpleEntry<String, Object>("descriptionModel", descriptionModel),
                new AbstractMap.SimpleEntry<String, Object>("format", format)
        ));
        return model;
    }

    public DmpModel importFileToDmp(@RequestBody FileEnvelopeModel fileEnvelope) {

        logger.debug(new MapLogEntry("importFileToDmp " + FileEnvelopeModel.class.getSimpleName()).And("fileEnvelope", fileEnvelope));

        DmpModel model = fileTransformerExecutor.importDmp(fileEnvelope);

        this.auditService.track(AuditableAction.FileTransformer_ImportFileToDmp, Map.ofEntries(
                new AbstractMap.SimpleEntry<String, Object>("fileEnvelope", fileEnvelope)
        ));
        return model;
    }

    public DescriptionModel importFileToDescription(@RequestBody FileEnvelopeModel fileEnvelope) {
        logger.debug(new MapLogEntry("importFileToDescription " + FileEnvelopeModel.class.getSimpleName()).And("fileEnvelope", fileEnvelope));

        DescriptionModel model = fileTransformerExecutor.importDescription(fileEnvelope);

        this.auditService.track(AuditableAction.FileTransformer_ImportFileToDescription, Map.ofEntries(
                new AbstractMap.SimpleEntry<String, Object>("importFileToDescription ", fileEnvelope)
        ));
        return model;
    }

    public FileTransformerConfiguration getSupportedFormats() {
        logger.debug(new MapLogEntry("getSupportedFormats"));

        FileTransformerConfiguration model = fileTransformerExecutor.getConfiguration();

        this.auditService.track(AuditableAction.FileTransformer_GetSupportedFormats);

        return model;
    }
}
